@@ -1,5 +1,6 @@
 import inspect
-from typing import Any, Callable, Dict, Sequence, Type, Union, no_type_check
+from enum import Enum
+from typing import Any, Callable, Dict, Optional, Sequence, Type, Union, no_type_check
 
 import anyio
 from sqlalchemy import inspect as sqlalchemy_inspect, select
@@ -22,7 +23,7 @@ from wtforms import (
 )
 from wtforms.fields.core import UnboundField
 
-from sqladmin.fields import QuerySelectField, QuerySelectMultipleField
+from sqladmin.fields import JSONField, QuerySelectField, QuerySelectMultipleField
 
 
 @no_type_check
@@ -48,15 +49,33 @@ class ModelConverterBase:
 
         self.converters = converters
 
-    def get_converter(self, column: Column) -> Callable:
+    def get_converter(
+        self, prop: Union[ColumnProperty, RelationshipProperty]
+    ) -> Callable:
+        if not isinstance(prop, ColumnProperty):
+            return self.converters[prop.direction.name]
+
+        column = prop.columns[0]
         types = inspect.getmro(type(column.type))
 
+        # Search by module + name
+        for col_type in types:
+            type_string = f"{col_type.__module__}.{col_type.__name__}"
+
+            if type_string in self.converters:
+                return self.converters[type_string]
+
         # Search by name
         for col_type in types:
             if col_type.__name__ in self.converters:
                 return self.converters[col_type.__name__]
 
-        raise Exception(
+            # Support for custom types like SQLModel which inherit TypeDecorator
+            if hasattr(col_type, "impl"):
+                if col_type.impl.__name__ in self.converters:  # type: ignore
+                    return self.converters[col_type.impl.__name__]  # type: ignore
+
+        raise Exception(  # pragma: nocover
             f"Could not find field converter for column {column.name} ({types[0]!r})."
         )
 
@@ -66,15 +85,24 @@ class ModelConverterBase:
         mapper: Mapper,
         prop: Union[ColumnProperty, RelationshipProperty],
         engine: Union[Engine, AsyncEngine],
+        field_args: Dict[str, Any] = None,
+        label: Optional[str] = None,
+        override: Optional[Type[Field]] = None,
     ) -> UnboundField:
-        kwargs: Dict = {
-            "validators": [],
-            "filters": [],
-            "default": None,
-            "description": prop.doc,
-        }
+        if field_args:
+            kwargs = field_args.copy()
+        else:
+            kwargs = {}
 
-        converter = None
+        kwargs: Dict[str, Any]
+        kwargs.setdefault("label", label)
+        kwargs.setdefault("validators", [])
+        kwargs.setdefault("filters", [])
+        kwargs.setdefault("default", None)
+        kwargs.setdefault("description", prop.doc)
+        kwargs.setdefault("render_kw", {"class": "form-control"})
+
+        converter = self.get_converter(prop)
         column = None
 
         if isinstance(prop, ColumnProperty):
@@ -105,8 +133,6 @@ class ModelConverterBase:
                 kwargs["validators"].append(validators.Optional())
             else:
                 kwargs["validators"].append(validators.InputRequired())
-
-            converter = self.get_converter(column)
         else:
             nullable = True
             for pair in prop.local_remote_pairs:
@@ -115,8 +141,9 @@ class ModelConverterBase:
 
             kwargs["allow_blank"] = nullable
 
-            pk = mapper.primary_key[0].name
-            stmt = select(prop.mapper.class_)
+            target_model = prop.mapper.class_
+            pk = sqlalchemy_inspect(target_model).primary_key[0].name
+            stmt = select(target_model)
 
             if isinstance(engine, Engine):
                 with Session(engine) as session:
@@ -135,9 +162,9 @@ class ModelConverterBase:
                     ]
                     kwargs["object_list"] = object_list
 
-            converter = self.converters[prop.direction.name]
-
-        assert converter is not None
+        if override is not None:
+            assert issubclass(override, Field)
+            return override(**kwargs)
 
         return converter(
             model=model, mapper=mapper, prop=prop, column=column, field_args=kwargs
@@ -165,6 +192,7 @@ class ModelConverter(ModelConverterBase):
 
     @converts("Boolean", "dialects.mssql.base.BIT")
     def conv_Boolean(self, field_args: Dict, **kwargs: Any) -> Field:
+        field_args["render_kw"]["class"] = "form-check-input"
         return BooleanField(**field_args)
 
     @converts("Date")
@@ -177,7 +205,12 @@ class ModelConverter(ModelConverterBase):
 
     @converts("Enum")
     def conv_Enum(self, column: Column, field_args: Dict, **kwargs: Any) -> Field:
-        field_args["choices"] = [(e, e) for e in column.type.enums]
+        available_choices = [(e, e) for e in column.type.enums]
+        accepted_values = [choice[0] for choice in available_choices]
+
+        field_args["choices"] = available_choices
+        field_args["validators"].append(validators.AnyOf(accepted_values))
+        field_args["coerce"] = lambda v: v.name if isinstance(v, Enum) else str(v)
         return SelectField(**field_args)
 
     @converts("Integer")  # includes BigInteger and SmallInteger
@@ -199,23 +232,27 @@ class ModelConverter(ModelConverterBase):
     #     field_args["validators"].append(validators.NumberRange(min=1901, max=2155))
     #     return StringField(**field_args)
 
-    # @converts("dialects.postgresql.base.INET")
-    # def conv_PGInet(self, field_args: Dict, **kwargs: Any) -> Field:
-    #     field_args.setdefault("label", "IP Address")
-    #     field_args["validators"].append(validators.IPAddress())
-    #     return StringField(**field_args)
+    @converts("sqlalchemy.dialects.postgresql.base.INET")
+    def conv_PGInet(self, field_args: Dict, **kwargs: Any) -> Field:
+        field_args.setdefault("label", "IP Address")
+        field_args["validators"].append(validators.IPAddress())
+        return StringField(**field_args)
 
-    # @converts("dialects.postgresql.base.MACADDR")
-    # def conv_PGMacaddr(self, field_args: Dict, **kwargs: Any) -> Field:
-    #     field_args.setdefault("label", "MAC Address")
-    #     field_args["validators"].append(validators.MacAddress())
-    #     return StringField(**field_args)
+    @converts("sqlalchemy.dialects.postgresql.base.MACADDR")
+    def conv_PGMacaddr(self, field_args: Dict, **kwargs: Any) -> Field:
+        field_args.setdefault("label", "MAC Address")
+        field_args["validators"].append(validators.MacAddress())
+        return StringField(**field_args)
 
-    # @converts("dialects.postgresql.base.UUID")
-    # def conv_PGUuid(self, field_args: Dict, **kwargs: Any) -> Field:
-    #     field_args.setdefault("label", "UUID")
-    #     field_args["validators"].append(validators.UUID())
-    #     return StringField(**field_args)
+    @converts("sqlalchemy.dialects.postgresql.base.UUID")
+    def conv_PgUuid(self, field_args: Dict, **kwargs: Any) -> Field:
+        field_args.setdefault("label", "UUID")
+        field_args["validators"].append(validators.UUID())
+        return StringField(**field_args)
+
+    @converts("JSON")
+    def convert_JSON(self, field_args: dict, **extra: Any) -> Field:
+        return JSONField(**field_args)
 
     @converts("MANYTOONE")
     def conv_ManyToOne(self, field_args: Dict, **kwargs: Any) -> Field:
@@ -231,10 +268,17 @@ async def get_model_form(
     engine: Union[Engine, AsyncEngine],
     only: Sequence[str] = None,
     exclude: Sequence[str] = None,
+    column_labels: Dict[str, str] = None,
+    form_args: Dict[str, Dict[str, Any]] = None,
+    form_class: Type[Form] = Form,
+    form_overrides: Dict[str, Dict[str, Type[Field]]] = None,
 ) -> Type[Form]:
     type_name = model.__name__ + "Form"
     converter = ModelConverter()
     mapper = sqlalchemy_inspect(model)
+    form_args = form_args or {}
+    column_labels = column_labels or {}
+    form_overrides = form_overrides or {}
 
     attributes = []
     for name, attr in mapper.attrs.items():
@@ -247,8 +291,13 @@ async def get_model_form(
 
     field_dict = {}
     for name, attr in attributes:
-        field = await converter.convert(model, mapper, attr, engine)
+        field_args = form_args.get(name, {})
+        label = column_labels.get(name, None)
+        override = form_overrides.get(name, None)
+        field = await converter.convert(
+            model, mapper, attr, engine, field_args, label, override
+        )
         if field is not None:
             field_dict[name] = field
 
-    return type(type_name, (Form,), field_dict)
+    return type(type_name, (form_class,), field_dict)
